/*
**   Tobit.e
**
**   ML estimation tobit model using nroptmum
**   Purpose: Illustration of use nroptmum
**   Author: Marius Ooms
**   Update for Gauss for Windows NT 3.2.35 and Gauss for Unix 3.2.35 : 12/7/2000,
**       Help on nroptmum available using Gauss browser after appropriate
**       construction of nroptmum library using the GAUSS-commands,
**       Be sure that you have the permission to update the library-folder
**       when you are running Gauss in a network-environment:
**          lib nroptmum nroptmum.dec
**          lib nroptmum nroptmum.ext
**          lib nroptmum nroptmum.src
**                    
*/
library pgraph; @,nroptmum@; @ nroptmum, make own library file using the lib command first! @
declare _x=1;      /* global data matrix */
declare _nulind=1; /* global data matrix */
declare _posind=1; /* global data matrix */
#include nroptmum.dec; @ not necessary if nroptmum library is made and used @
#include nroptmum.ext; @ not necessary if nroptmum library is made and used @
#include nroptmum.src; @ not necessary if nroptmum library is made and used @
fname=envget("gauss_cfg");
" Folder where active gauss.cfg is residing and lib_path is chosen: "$+fname;
outfile = "tobit.out";@ name output-file @
datafile = "kleding"; @ name GAUSS data-file, case sensitive in UNIX!! @
   _nroutpu = 2;      @ 0: no intermiate output, @
   					  @	1: to output-file and screen @
   					  @ 2: intermediate output to screen only (not so much in Unix!) @
   _nrgtol  = 1e-5;   @ convergence parameters @
   _nrftol  = 1e-15;  @ conv. function for dataset kleding, optmimal for alg 1 @
   _nrpmax  = 1e+6;   @ maximum absolute value parameter @
   _nrmiter = 1000;   @ maximum number of iterations @
   _nrmtime = 300;    @ maximum time in seconds @
   _nralgr  = 2;      @ Algorithm, 1: amoeba, 2: BFGS, 4: Powell, 6: FRPR @
/*
**  mlltobi
**  purpose :  compute -1*loglikelihood for a tobit model
**  format  :  {mll}   = mlltobi(p)
**  input   :  p       = (n x k) vector parameter values for (n-1) explanatory
**                       variables and ln(variance error term)
**  output  :  mll     =  (k x 1) -1 * log likelihood
**  globals :  _x      =  t x n data matrix, first column dependent variable
**             _nulind =     t0 x 1 indexes for which _x[.,1] == 0
**             _posind = (t-t0) x 1 indexes for which _x[.,1]  > 0
**  date    :  9/9/96
**  author  :  Marius Ooms
**  Remark: This is NOT the best parameterization for optimization!
**   See
**   Maddala, G.S. (1983), Limited-dependent and qualitative variables in
**               econometrics, Cambridge University Press, Chapter 6.
*/
proc mlltobi(p);
    local eps, s2, mll, k, n, t1;
       k    = cols(p);
       n    = rows(p);
       s2   = exp(p[n,.]);
       eps  = _x[.,2:n]*p[1:n-1,.];
       t1   = rows(_x)-rows(_nulind);
       mll  = sumc(ln(cdfnc(eps[_nulind,.]./sqrt(s2))));
       eps  = _x[_posind,1]-eps[_posind,.];
       mll  = mll - sumc(eps.*eps)./(2*s2')-(t1/2).*(ln(2*pi)+p[n,.]');
    retp(-mll);
endp;
outfile = "tobit.out";
datafile = "kleding";
datafiln = datafile$+"n";
outwidth 160;
output file = ^outfile reset;
 #IFUNIX
 load _x = ^datafile;                @ load data from GAUSS-matrix file (.fmt) @
 load parnm = ^datafiln;             @ load data from GAUSS-matrix file (.fmt) @
 #else
 _x = loadd(datafile);                 @ load data from GAUSS-DATA file (.dat) @
 parnm = getname(datafile);            @   ,,      from GAUSS-DATA file (.dht) @
 #endif
 call saved(_x,"klednieu",parnm); @ save data in contemporary local GAUSS format @
 scalex= 1~0.1~10~100 ;                                 @ scaling   @
 _x = _x.*scalex;                                       @ scaling   @
 n=cols(_x);                                         @ no. of variables    @
 t=rows(_x);                                         @ no. of observations @
 _nrparnm = parnm[2:n]|"ln(sig2)";                   @ parameter names     @
 _nulind = indexcat(_x[.,1],0); @ indexes zero observations dependent variable @
 _posind = setdif(seqa(1,1,t),_nulind,1);   @ indexes other observations   @
 b = _x[_posind,1]/_x[_posind,2:n];         @ starting value for "beta"    @
 s2 = sumc((_x[_posind,1]-_x[_posind,2:n]*b)^2)/rows(_posind);
 s2 = ln(s2);                             @ starting value for "sigma-squared" @
 p0=b|s2;
 date1=date;                              @ timing @
 time1=time;                              @  ,,    @
 {p,f0,g,retcode} = nroptmum(&mlltobi,p0);@ minimization @
 varp = diag(invpd(_nrfhess));         @ variance parameter estimates @
 #ifUNIX
 @WinClear(1);@ @ Useful if X Windows version used @
 call WinSetActive(1);
 #else
 cls;                                  @ clear screen does not work for Unix @
 pqgwin auto;
 #endif
 date2=date;
format /rz 5,4;
plotfunc(&mlltobi,_nrparnm,p,varp); @ plot -log-likelihood around minimum @
/*  Output  */
"Date and time:" (rev(date1[1:3])|(time1[1:3]))';
"Data-file: "$+datafile;
"Algorithm: " _nralgr;
format /ro 10,8;
nrerror(retcode);
"Computation time: " ethsec(date1,date2)/100;
format /ro 20,16;
"Function:   " -f0; /* loglikelihood */
format /ro 11,6;
"Gradient:      " g';
"Diag. hessian: " diag(_nrfhess)';
"Scaling:       " scalex[.,2:n];
"";
"Dependent variable: " $parnm[1];
printest(_nrparnm,p,varp);
output off;
"Results in "$+outfile;
